from pydantic import BaseModel

from deciphon_snap.match_list import MatchList

__all__ = ["Prod"]


class Prod(BaseModel):
    seq_id: int
    profile: str
    abc: str
    alt: float
    null: float
    evalue: float
    match_list: MatchList



from typing import List

from pydantic import BaseModel

from deciphon_snap.match import Match

__all__ = ["MatchList"]


class MatchList(BaseModel):
    __root__: List[Match]

    @classmethod
    def from_string(cls, x: str):
        return cls.parse_obj([Match.from_string(i) for i in x.split(";")])

    def __len__(self):
        return len(self.__root__)

    def __getitem__(self, i):
        return self.__root__[i]

    def __iter__(self):
        return iter(self.__root__)

    def __str__(self):
        return " ".join(str(i) for i in self.__root__)



from typing import List

from pydantic import BaseModel

from deciphon_snap.prod import Prod

__all__ = ["ProdList"]


class ProdList(BaseModel):
    __root__: List[Prod]

    def __len__(self):
        return len(self.__root__)

    def __getitem__(self, i):
        return self.__root__[i]

    def __iter__(self):
        return iter(self.__root__)



from typing import List

from pydantic import BaseModel

from deciphon_snap.match import Match

__all__ = ["Hit"]


class Hit(BaseModel):
    id: int
    name: str
    prod_id: int
    evalue: float
    matchs: List[Match]
    feature_start: int = 0
    feature_end: int = 0



from __future__ import annotations
from pydantic import BaseModel


__all__ = ["Match"]


class Match(BaseModel):
    query: str
    state: str
    codon: str
    amino: str

    @classmethod
    def from_string(cls, x: str):
        y = x.split(",", 3)
        return cls(query=y[0], state=y[1], codon=y[2], amino=y[3])

    def __str__(self):
        query = self.query if len(self.query) > 0 else "∅"
        state = self.state
        codon = self.codon if len(self.codon) > 0 else "∅"
        amino = self.amino if len(self.amino) > 0 else "∅"
        return f"({query},{state},{codon},{amino})"



from __future__ import annotations

import csv
from typing import List

import prettytable as pt

from deciphon_snap.match_list import MatchList
from deciphon_snap.prod import Prod
from deciphon_snap.prod_list import ProdList
from deciphon_snap.shorten import shorten
from deciphon_snap.stringify import stringify

__all__ = ["SnapFile"]


class SnapFile:
    def __init__(self, filesystem):
        self._fs = filesystem

        files = self._fs.ls("/")
        assert len(files) == 1

        root_dir = files[0]["filename"][:-1]
        assert self._fs.isdir(root_dir)
        prod_file = f"{root_dir}/products.tsv"

        with self._fs.open(prod_file, "rb") as file:
            self._prods = read_products(file)

    @property
    def products(self):
        return self._prods

    def __str__(self):
        fields = list(Prod.schema()["properties"].keys())

        num_fields = len(fields)
        prods = [[getattr(x, i) for i in fields] for x in self.products]
        num_products = len(prods)
        if num_products >= 10:
            prods = prods[:4] + [["…"] * num_fields] + prods[-4:]

        x = pt.PrettyTable()
        x.set_style(pt.SINGLE_BORDER)
        x.field_names = fields
        x.align = "l"
        for prod in prods:
            x.add_row([shorten(i) for i in prod])

        header = f"shape: ({num_products}, {num_fields})"
        return header + "\n" + x.get_string()


def read_products(file):
    prods: List[Prod] = []
    for row in csv.DictReader((stringify(i) for i in file), delimiter="\t"):
        prods.append(
            Prod(
                seq_id=int(row["seq_id"]),
                profile=str(row["profile"]),
                abc=str(row["abc"]),
                alt=float(row["alt"]),
                null=float(row["null"]),
                evalue=float(row["evalue"]),
                match_list=MatchList.from_string(str(row["match"])),
            )
        )
    return ProdList.parse_obj(prods)



from deciphon_snap.match_list import MatchList

__all__ = ["shorten"]


def shorten(x: str | int | float | MatchList, size: int = 32):
    if isinstance(x, float):
        return f"{x:.9g}"
    if isinstance(x, int):
        return str(x)
    if isinstance(x, MatchList):
        x = str(x)
    return x[:size] + "…" if len(x) > size else x
